clear

# set variables
VERSION=`cat version.txt`
NAMESTRING="commit_${VERSION}"
OUTNAMESTRING="${NAMESTRING}"
SHOULDCLEAN="0"
SHOULDFLAT="0"
SHOULDTAR="0"
SHOULDDEB="0"
SHOULDRPM="0"
SHOULDDISABLEUPDATE="0"
SHOULDSOURCE="0"

# detect the CLI flags
while [ "$1" != "" ]; do
  if [ "$1" == "-f" ]; then
    SHOULDFLAT="1"
  elif [ "$1" == "-c" ]; then
    SHOULDCLEAN="1"
  elif [ "$1" == "-t" ]; then
    SHOULDTAR="1"
  elif [ "$1" == "-d" ]; then
    SHOULDDEB="1"
  elif [ "$1" == "-r" ]; then
    SHOULDRPM="1"
  elif [ "$1" == "-n" ]; then
    SHOULDDISABLEUPDATE="1"
  elif [ "$1" == "-s" ]; then
    SHOULDSOURCE="1"
  fi
  shift
done

# cleanup
printf "Doing some cleanup...\n"
rm -rf dist
if [ "$SHOULDCLEAN" == "1" ]; then
  rm -rf build
fi

# make directories
printf "Making required directories...\n"
mkdir -p dist
mkdir -p build/stage/commit/commit/assets/icons
mkdir -p build/stage/commit/commit/assets/sound
mkdir -p build/stage/commit/commit/assets/toolbars
mkdir -p build/stage/commit/commit/includes
mkdir -p build/stage/commit/commit/ui

# copy app files
printf "Copying files to staging area...\n"

sed "s/\$VERSION/$VERSION/g" commit/assets/installer/lin/com.intersanity.commit.appdata.xml > \
  build/stage/com.intersanity.commit.appdata.xml

cp commit/assets/installer/lin/com.intersanity.commit.xml build/stage
cp commit.py *.md version.txt build/stage/commit
cp commit/*.py build/stage/commit/commit
cp -r commit/locale build/stage/commit/commit
cp commit/assets/*.png build/stage/commit/commit/assets
cp -r commit/includes/*.py build/stage/commit/commit/includes
cp -r commit/ui/*.py build/stage/commit/commit/ui
cp -r commit/assets/icons/*.png build/stage/commit/commit/assets/icons
cp -r commit/assets/sound/*.wav build/stage/commit/commit/assets/sound
cp -r commit/assets/toolbars/*fbk*.png commit/assets/toolbars/*sb_dot*.png build/stage/commit/commit/assets/toolbars

cp commit/assets/icons/appicon.svg build/stage/
cp commit/assets/icons/docicon.svg build/stage/
cp commit/assets/icons/appicon*.png build/stage/
cp commit/assets/icons/docicon*.png build/stage/

# if we need to disable the update feature
if [ "$SHOULDDISABLEUPDATE" == "1" ]; then
  printf "Disabling auto-update system...\n"
  touch build/stage/commit/noupdate
  OUTNAMESTRING="${NAMESTRING}_noupdate"
fi

if [ "$SHOULDFLAT" == "1" ]; then
  printf "Downloading Pandoc for bundling...\n"
  python3 ./commit/assets/installer/lin/flatpak/get_pandoc.py
  mv ./build/stage/copyright.pandoc ./build/stage/pandoc-copyright.txt

  # prep work
  printf "Preparing the Flatpak staging area and copying files...\n"
  mkdir -p build/flatpak/build
  mkdir -p build/flatpak/stage
  mkdir -p build/flatpak/repo

  cp -r build/stage/* build/flatpak/stage/

  cp commit/assets/installer/lin/flatpak/python3-requirements.yaml build/flatpak/stage/
  cp commit/assets/installer/lin/flatpak/com.intersanity.commit.yaml build/flatpak/stage/

  sed "s/\$PATH/\/app\/share/g" commit/assets/installer/lin/commit.sh > \
    build/flatpak/stage/commit.sh
  chmod +x build/flatpak/stage/commit.sh

  sed "s/\$PATH/\/app\/sbin/g" commit/assets/installer/lin/com.intersanity.commit.desktop > \
    build/flatpak/stage/com.intersanity.commit.desktop

  # install correct SDKs
  printf "Installing the correct Flatpak platform and SDK...\n"
  # flatpak install -y --noninteractive flathub org.freedesktop.Platform//21.08 org.freedesktop.Sdk//21.08
  flatpak install -y --noninteractive flathub org.gnome.Platform//43 org.gnome.Sdk//43

  printf "Building the Flatpak package...\n"
  cd build/flatpak/stage/
  flatpak-builder --repo='../repo' --force-clean '../build' com.intersanity.commit.yaml
  cd ../../../
  flatpak build-bundle build/flatpak/repo dist/$OUTNAMESTRING.flatpak com.intersanity.commit
fi

if [ "$SHOULDDEB" == "1" ] || [ "$SHOULDRPM" == "1" ]; then
  printf "Organizing files for the DEB and/or RPM...\n"
  mkdir -p build/deb/usr/sbin
  mkdir -p build/deb/usr/share/applications
  mkdir -p build/deb/usr/share/doc/commit/
  mkdir -p build/deb/usr/share/metainfo/
  mkdir -p build/deb/usr/share/mime/packages
  mkdir -p build/deb/usr/share/icons/hicolor/scalable/mimetypes/
  mkdir -p build/deb/usr/share/icons/hicolor/scalable/apps/
  mkdir -p build/deb/usr/share/icons/hicolor/64x64/mimetypes/
  mkdir -p build/deb/usr/share/icons/hicolor/64x64/apps/
  mkdir -p build/deb/usr/share/icons/locolor/64x64/apps/
  mkdir -p build/deb/DEBIAN

  cp commit/assets/icons/appicon.svg build/deb/usr/share/icons/hicolor/scalable/apps/com.intersanity.commit.svg
  cp commit/assets/icons/appicon64.png build/deb/usr/share/icons/hicolor/64x64/apps/com.intersanity.commit.png
  cp commit/assets/icons/appicon64.png build/deb/usr/share/icons/locolor/64x64/apps/com.intersanity.commit.png
  cp commit/assets/icons/docicon.svg build/deb/usr/share/icons/hicolor/scalable/mimetypes/com.intersanity.commit-text.svg
  cp commit/assets/icons/docicon64.png build/deb/usr/share/icons/hicolor/64x64/mimetypes/com.intersanity.commit-text.png
  cp build/stage/com.intersanity.commit.appdata.xml build/deb/usr/share/metainfo/
  cp build/stage/com.intersanity.commit.xml build/deb/usr/share/mime/packages/
  cp build/stage/commit/LICENSE.md build/deb/usr/share/doc/commit/copyright

  # copy everything else from stage
  cp -r build/stage/commit build/deb/usr/share/

  sed "s/\$PATH/\/usr\/share/g" commit/assets/installer/lin/commit.sh > \
    build/deb/usr/sbin/commit

  sed "s/\$PATH/\/usr\/sbin/g" commit/assets/installer/lin/com.intersanity.commit.desktop > \
    build/deb/usr/share/applications/com.intersanity.commit.desktop

fi

if [ "$SHOULDDEB" == "1" ]; then
  printf "Copying DEB-specific files...\n"

  sed "s/\$VERSION/$VERSION/g" commit/assets/installer/lin/debrpm/deb-commit.control > \
    build/deb/DEBIAN/control

  CURRENTDATE=`date --rfc-email`
  sed "s/\$VERSION/$VERSION/g;s/\$CURRENTDATE/$CURRENTDATE/g" \
    commit/assets/installer/lin/debrpm/deb-commit.changelog > \
    build/deb/usr/share/doc/commit/changelog.Debian
  gzip -f9 build/deb/usr/share/doc/commit/changelog.Debian

  printf "Setting the proper permissions...\n"
  find build/deb -type d -exec chmod 0755 {} +
  find build/deb -type f -exec chmod 0644 {} +
  chmod 0755 build/deb/usr/sbin/commit

  printf "Building the DEB...\n"
  # read -s -n 1 -p "Press any key to continue...."
  dpkg-deb -v -Z gzip --root-owner-group --build build/deb dist
fi

# create RPM file
if [ "$SHOULDRPM" == "1" ]; then
  printf "Organizing files for the RPM...\n"
  rpmdev-setuptree
  mkdir -p build/rpm/
  cp -r build/deb/usr build/rpm/
  if [ "$SHOULDDEB" != "1" ]; then
    rm -rf build/deb
  fi
  rm -f build/rpm/usr/share/doc/commit/changelog.Debian.gz
  rm -rf ~/rpmbuild/BUILD/usr
  cp -r build/rpm/usr ~/rpmbuild/BUILD

  CURRENTDATE=`date +"%a %b %d %Y"`
  sed "s/\$VERSION/$VERSION/g;s/\$CURRENTDATE/$CURRENTDATE/g" \
    commit/assets/installer/lin/debrpm/rpm-commit.spec > \
    build/rpm/commit.spec

  printf "Building the RPM package...\n"
  rpmbuild -bb --quiet ./build/rpm/commit.spec
  mv ~/rpmbuild/RPMS/noarch/commit-*.rpm ./dist/
  rm -rf ~/rpmbuild/BUILD/usr
fi

# create the TAR.GZ for manual distribution
if [ "$SHOULDTAR" == "1" ]; then
  printf "Creating a TAR.GZ for manual distribution...\n"
  mkdir build/tarball
  cp -r ./build/stage/commit build/tarball/commit
  cat << EOF > build/tarball/commit/commit
#!/bin/sh
python3 commit.py "$1"
EOF
  chmod +x build/tarball/commit/commit
  tar -C ./build/tarball -czvf ./dist/$OUTNAMESTRING.linux.tar.gz commit
fi

# create a source tarball
if [ "$SHOULDSOURCE" == "1" ]; then
  printf "Creating the source tarball...\n"
  tar --exclude='.git' --exclude='.idea' --exclude='__pycache__' --exclude='./build' --exclude='./dist' --exclude='./venv' -czvf ./dist/commit_${VERSION}_source.tar.gz ./
fi

# done
printf "Our work here is done.\n"
