import pathlib          # Python Pathlib - https://docs.python.org/3/library/pathlib.html
import sys
from setuptools import setup

version = pathlib.Path("./version.txt").read_text()
winver = pathlib.Path("./winver.txt").read_text()

app = ['commit.py']
data_files = [
    (
        "", [
            "version.txt",
            "CREDITS.md",
            "LICENSE.md"
        ]
    ),
    (
        "commit/assets", [
            "commit/assets/commit-splash.png"
        ]
    ),
    (
        "commit/assets/icons", [
            "commit/assets/icons/appicon16.png",
            "commit/assets/icons/appicon24.png",
            "commit/assets/icons/appicon32.png",
            "commit/assets/icons/appicon48.png",
            "commit/assets/icons/appicon64.png",
            "commit/assets/icons/appicon96.png",
            "commit/assets/icons/appicon128.png",
            "commit/assets/icons/appicon256.png",
            "commit/assets/icons/appicon512.png",
            "commit/assets/icons/appicon1024.png"
        ]
    ),
    (
        "commit/assets/sound", [
            "commit/assets/sound/commit.wav",
            "commit/assets/sound/pull.wav"
        ]
    ),
    (
        "commit/assets/toolbars", [
            "commit/assets/toolbars/fbk_sb_menu.png",
            "commit/assets/toolbars/fbk_sb_meta.png",
            "commit/assets/toolbars/fbk_sb_ratio.png",
            "commit/assets/toolbars/fbk_sb_theme.png",
            "commit/assets/toolbars/fbk_tb_chevron_down.png",
            "commit/assets/toolbars/fbk_tb_chevron_up.png",
            "commit/assets/toolbars/fbk_tb_fullscreen.png",
            "commit/assets/toolbars/fbk_tb_fullscreen_back.png",
            "commit/assets/toolbars/fbk_tb_menu.png",
            "commit/assets/toolbars/fbk_tb_meta.png",
            "commit/assets/toolbars/fbk_tb_prefs.png",
            "commit/assets/toolbars/fbk_tb_quit.png",
            "commit/assets/toolbars/inv_fbk_sb_menu.png",
            "commit/assets/toolbars/inv_fbk_sb_meta.png",
            "commit/assets/toolbars/inv_fbk_sb_ratio.png",
            "commit/assets/toolbars/inv_fbk_sb_theme.png",
            "commit/assets/toolbars/inv_fbk_tb_chevron_down.png",
            "commit/assets/toolbars/inv_fbk_tb_chevron_up.png",
            "commit/assets/toolbars/inv_fbk_tb_fullscreen.png",
            "commit/assets/toolbars/inv_fbk_tb_fullscreen_back.png",
            "commit/assets/toolbars/inv_fbk_tb_menu.png",
            "commit/assets/toolbars/inv_fbk_tb_meta.png",
            "commit/assets/toolbars/inv_fbk_tb_prefs.png",
            "commit/assets/toolbars/inv_fbk_tb_quit.png",
            "commit/assets/toolbars/inv_sb_dot_saved.png",
            "commit/assets/toolbars/inv_sb_dot_unsaved.png",
            "commit/assets/toolbars/sb_dot_saved.png",
            "commit/assets/toolbars/sb_dot_unsaved.png"
        ]
    )
]

# conditionals
if sys.platform == "darwin":
    import py2app
    data_files.append(
        (
            "commit/assets/toolbars/", [
                "commit/assets/toolbars/inv_mac_tb_chevron_down.png",
                "commit/assets/toolbars/inv_mac_tb_chevron_up.png",
                "commit/assets/toolbars/inv_mac_tb_fullscreen.png",
                "commit/assets/toolbars/inv_mac_tb_fullscreen_back.png",
                "commit/assets/toolbars/inv_mac_tb_menu.png",
                "commit/assets/toolbars/inv_mac_tb_meta.png",
                "commit/assets/toolbars/inv_mac_tb_prefs.png",
                "commit/assets/toolbars/inv_mac_tb_quit.png",
                "commit/assets/toolbars/mac_tb_chevron_down.png",
                "commit/assets/toolbars/mac_tb_chevron_up.png",
                "commit/assets/toolbars/mac_tb_fullscreen.png",
                "commit/assets/toolbars/mac_tb_fullscreen_back.png",
                "commit/assets/toolbars/mac_tb_menu.png",
                "commit/assets/toolbars/mac_tb_meta.png",
                "commit/assets/toolbars/mac_tb_prefs.png",
                "commit/assets/toolbars/mac_tb_quit.png"
            ]
        )
    )
    data_files.append(
        (
            "", [  # root Contents dir
                "commit/assets/icons/docicon.icns"
            ]
        )
    )
    # setup values
    setup(
        app=app,
        name="Commit",
        version=version,
        description="the Advanced Modern Typewriter",
        author="Intersanity Enterprises and contributors",
        data_files=data_files,
        options={'py2app': {
            "plist": {
                "CFBundleIdentifier": "com.intersanity.commit",
                "CFBundleDocumentTypes": [
                    {
                        "CFBundleTypeName": "Commit Text File",
                        "CFBundleTypeRole": "Editor",
                        "LSHandlerRank": "Owner",
                        "LSItemContentTypes": ["com.intersanity.commit"]
                    }
                ],
                "UTExportedTypeDeclarations": [
                    {
                        "UTTypeIdentifier": "com.intersanity.commit",
                        "UTTypeDescription": "Commit Text File",
                        "UTTypeConformsTo": [
                            "public.plain-text"
                        ],
                        "UTTypeTagSpecification": {
                            "public.filename-extension": ["ctxt"],
                            "public.mime-type": ["text/com.intersanity.commit"]
                        },
                        "UTTypeIconFile": "docicon.icns",
                        "UTTypeReferenceURL": "https://www.intersanity.com/software/commit/manual/"
                    }
                ],
                "NSHumanReadableCopyright": "(c) 2022-2023 Intersanity Enterprises and contributors"
            }
        }},
        setup_requires=['py2app']
    )

elif sys.platform == "win32":
    # import py2exe
    import re
    from py2exe import freeze
    data_files.append(
        (
            "commit/assets/toolbars/", [
                "commit/assets/toolbars/inv_win_tb_chevron_down.png",
                "commit/assets/toolbars/inv_win_tb_chevron_up.png",
                "commit/assets/toolbars/inv_win_tb_fullscreen.png",
                "commit/assets/toolbars/inv_win_tb_fullscreen_back.png",
                "commit/assets/toolbars/inv_win_tb_menu.png",
                "commit/assets/toolbars/inv_win_tb_meta.png",
                "commit/assets/toolbars/inv_win_tb_prefs.png",
                "commit/assets/toolbars/inv_win_tb_quit.png",
                "commit/assets/toolbars/win_tb_chevron_down.png",
                "commit/assets/toolbars/win_tb_chevron_up.png",
                "commit/assets/toolbars/win_tb_fullscreen.png",
                "commit/assets/toolbars/win_tb_fullscreen_back.png",
                "commit/assets/toolbars/win_tb_menu.png",
                "commit/assets/toolbars/win_tb_meta.png",
                "commit/assets/toolbars/win_tb_prefs.png",
                "commit/assets/toolbars/win_tb_quit.png",
                "commit/assets/toolbars/winscroll-dark-down.png",
                "commit/assets/toolbars/winscroll-dark-up.png"
            ]
        )
    )
    freeze(
        windows=[
            {
                "script": app[0],
                "icon_resources": [
                    (0, "commit/assets/icons/appicon.ico"),
                    (1, "commit/assets/icons/docicon.ico")
                ]
            }
        ],
        data_files=data_files,
        options={
            "includes": ["wx._xml", 'lxml._elementpath', 'html.parser'],
            "dist_dir": "dist\commit"
        },
        version_info={
            "version": winver,
            "product_name": "Commit",
            "description": "Commit: the Advanced Modern Typewriter",
            "company_name": "Intersanity Enterprises",
            "copyright": "(c) 2022-2023 Intersanity Enterprises and contributors"
        }
    )

else:
    setup(
        app=app,
        name="Commit",
        version=version,
        description="the Advanced Modern Typewriter",
        author="Intersanity Enterprises and contributors",
        data_files=data_files
    )
